@@ -2,12 +2,19 @@ from __future__ import division, absolute_import
 
 import numpy.testing
 
-from refnx._lib.util import (TemporaryDirectory, preserve_cwd, flatten, unique,
+from refnx._lib.util import (TemporaryDirectory, preserve_cwd,
                              possibly_open_file, possibly_create_pool)
 from refnx._lib._numdiff import approx_hess2
 
 from refnx._lib._testutils import PytestTester
 
+try:
+    from refnx._lib._cutil import c_unique as unique
+    from refnx._lib._cutil import c_flatten as flatten
+except ImportError:
+    from refnx._lib.util import unique, flatten
+
+
 test = PytestTester(__name__)
 del PytestTester
 
